package gentryfilename;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileView;

import fr.cnes.genius.constraints.GConstraint;
import fr.cnes.genius.exception.GException;
import fr.cnes.genius.exception.GIntervalException;
import fr.cnes.genius.highLevel.GEntryString;
import fr.cnes.genius.interval.GIntervalEmptyString;
import fr.cnes.genius.lowLevel.GButton;
import fr.cnes.genius.lowLevel.GContainer;
import fr.cnes.genius.lowLevel.GFileChooser;
import fr.cnes.genius.main.GDisplay;
import fr.cnes.genius.main.GEvent;
import fr.cnes.genius.main.GListener;
import fr.cnes.genius.main.GReadWrite;

/**
 * Class to define a widget for searching file names giving its relative or absolute names.
 * @author goesterjf
 *
 */
public class GEntryFileName extends GContainer implements GDisplay, GReadWrite, GListener {
    
    /** user.dir key */
    private static final String USER_DIR = "user.dir";
    /** Regular expression for \ */
    private static final String REG_ANTISLASH = "\\\\";
    /** Regular expression for / */
    private static final String REG_SLASH = "\\/";
    /** String "/" */
    private static final String SLASH = "/";
    /** ":" character */
    private static final String SEMI_COL = ":";

    /** Button for giving the file name */
    private final GEntryString widNameFile;
    /** Button for searching the file */
    private final GButton butFile;
    
    /** Current directory path for searching files */
    private String currentDirPath;
    
    /** Title of the browser panel */
    private final String title;
    /** Description of the type of files */
    private final String description;
    /** Suffix of the files to search */
    private final String suffix;
    
    /** Relative name if true */
    private final boolean isRelative;

    /**
     * Constructor.
     * @param isRelative       Relative name if true (absolute else)
     * @param label            label
     * @param iniVal           initial value
     * @param varName          variable name in the XML file
     * @param width            entry area width
     * @param titleFileChooser title of the file chooser panel
     * @param description      description of the type of files to search
     * @param suffix           suffix of the files
     */
    public GEntryFileName (final boolean isRelative, final String label, final String iniVal, final String varName, final int width,
        final String titleFileChooser, final String description, final String suffix) {
        
        this.isRelative = isRelative;
        this.title = titleFileChooser;
        this.description = description;
        this.suffix = suffix;
        
        widNameFile = new GEntryString(label, iniVal);
        widNameFile.setNameInConfigFile(varName);
        widNameFile.setInnerDescendantConstraint(new GConstraint(GConstraint.newline(false),
                GConstraint.width(width), GConstraint.split(2)), 1);
        try {
            widNameFile.addGIntervalEmptyString(new GIntervalEmptyString());
        } catch ( GIntervalException err ) {
            // Will never occur
        }
        
        butFile = new GButton("...");
        butFile.setConstraint(new GConstraint(GConstraint.newline(false)));

        currentDirPath = ".";

    }

    @Override
    public void display () throws GException {
        generic();
    }

    @Override
    public void generic () throws GException {
        put(widNameFile);
        put(butFile);
    }

    @Override
    public void read () throws GException {
        generic();
        testOnFile();
    }

    @Override
    public void write () throws GException {
        generic();
    }

    @Override
    public void before (GEvent e) throws GException {
        // Nothing to do
    }
    
    @Override
    public void after (GEvent e) throws GException {
        
        if ( e.contains(butFile)) {
            
            final File file = selectFileWithChooser(JFileChooser.FILES_ONLY, currentDirPath, false,
                    title, description, suffix);
            if ( file != null ) {
                currentDirPath = file.getParent();
                if ( isRelative ) {
                    widNameFile.setValue(correctFileName(file.getAbsolutePath(), System.getProperty(USER_DIR)));
                } else {
                    widNameFile.setValue(file.getAbsolutePath().replaceAll(REG_ANTISLASH, REG_SLASH));
                }
            }

        }
        
        if ( e.contains(widNameFile)) {
            testOnFile();
        }
        
    }
    
    private final void testOnFile() {
        
        final File file = new File(widNameFile.getValue());
        if ( file.exists() ) {
            if ( isRelative ) {
                widNameFile.setValue(correctFileName(widNameFile.getValue(), System.getProperty(USER_DIR)));
            } else {
                widNameFile.setValue(widNameFile.getValue().replaceAll(REG_ANTISLASH, REG_SLASH));
            }
        } else {
            widNameFile.setValue("");
        }

    }
    
    /**
     * Opens a JFileChooser to select a file.
     * @param type        JFileChooser type
     * @param iniPath     initial directory path
     * @param blockDir    block inside the directory path if true
     * @param title       sub window title
     * @param description description
     * @param suffix      file suffix (may be null)
     * @return File object selected (may be null)
     */
    public static File selectFileWithChooser(final int type, final String iniPath, final boolean blockDir,
        final String title, final String description, final String suffix) {
        
        final File iniFile = new File(iniPath);
        final GFileChooser fileChooser = new GFileChooser(iniFile);
        if ( blockDir ) {
            fileChooser.setFileView(new FileView() {
                @Override
                public Boolean isTraversable(File f) {
                    return iniFile.equals(f);
                }
            });
        }
        fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        fileChooser.setFileSelectionMode(type);
        fileChooser.setApproveButtonText("Load");
        fileChooser.setDialogTitle(title);
        
        final FileFilter filter = new FileFilter() {
            
            @Override
            public boolean accept (File file) {
                boolean rep = false;
                if ( file.isDirectory() ) {
                    rep = true;
                } else {
                    if ( suffix != null ) {
                        if ( (file.getName().endsWith(suffix.toLowerCase())) || (file.getName().endsWith(suffix.toUpperCase())) ) {
                            rep = true;
                        }
                    } else {
                        rep = true;
                    }
                }
                return rep;
            }

            @Override
            public String getDescription () {
                return description;
            }
        };
        fileChooser.setFileFilter(filter);
    
        File selectedFile = null;
        if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
            selectedFile = fileChooser.getSelectedFile();
        }
        
        return selectedFile;
        
    }

    /**
     * Method to extract the relative name with "/" rather than "\" and to test also if the file exists !
     * @param iniName      initial file full name
     * @param iniLocalDir  local directory path
     * @return             relative name with "/" ... or "" if the file dose not exist
     */
    public static String correctFileName (final String iniName, final String iniLocalDir) {
        
        final String name = iniName.replaceAll(REG_ANTISLASH, REG_SLASH);
        String finalName = "";
        
        // Getting the path with "/"
        final File ldir = new File(iniLocalDir);
        final String localDir = ldir.getAbsolutePath().replaceAll(REG_ANTISLASH, REG_SLASH);
        
        if ( name.startsWith(SLASH) || iniName.contains(SEMI_COL)) {
            
            // Splitting the directories
            final String[] localDirSplit = localDir.split(SLASH);
            
            boolean continueRel = true;
            if ( iniName.contains(SEMI_COL) ) {
                
                // Specific case of a Windows network
                
                final String[] nameSplit = name.split(SLASH);
                final String nameNet = nameSplit[0];                
                final String localNet = localDirSplit[0];
                
                if ( !nameNet.equals(localNet) ) {
                    // The path has to be kept as an absolute one as we are not on the same server
                    finalName = name;
                    continueRel = false; 
                }
                
            }
            
            if ( continueRel ) {
                
                // Finding the common root
                String relIniName = name;
                String relLocalDir = localDir;
                int numRep = localDirSplit.length;
                for (int i = 0; i < localDirSplit.length; i++) {
                    if ( relIniName.startsWith(localDirSplit[i]) ) {
                        numRep--;
                        relIniName = relIniName.substring(localDirSplit[i].length());
                        if ( relIniName.startsWith(SLASH) ) {
                            relIniName = relIniName.substring(1);
                        }
                        relLocalDir = relLocalDir.substring(localDirSplit[i].length());
                        if ( relLocalDir.startsWith(SLASH) ) {
                            relLocalDir = relLocalDir.substring(1);
                        }
                    } else {
                        break;
                    }
                }
                
                // Final relative path
                for (int i = 0; i < numRep; i++) {
                    finalName = finalName + "../";
                }
                finalName = finalName + relIniName;
                
            }            

        } else  {
            
            // The path is already relative
            finalName = name;

        }
        
        // Test if the file exists !
        final File file = new File(localDir + SLASH + finalName);
        if ( ! file.exists() ) {
            finalName = "";
        }
        
        return finalName;
        
    }

}
